﻿#pragma once

#include "Common\StepTimer.h"
#include "Common\DeviceResources.h"
#include "Content\CharacterFormattingDemoRenderer.h"

// Renders Direct2D and 3D content on the screen.
namespace CharacterFormattingDemo
{
    class CharacterFormattingDemoMain : public DX::IDeviceNotify
    {
    public:
        CharacterFormattingDemoMain(const std::shared_ptr<DX::DeviceResources>& deviceResources);
        ~CharacterFormattingDemoMain();
        void CreateWindowSizeDependentResources();
        void StartTracking() {  }
        void TrackingUpdate(float positionX) { m_pointerLocationX = positionX; }
        void StopTracking() {  }
        bool IsTracking() { return false; }
        void StartRenderLoop();
        void StopRenderLoop();
        Concurrency::critical_section& GetCriticalSection() { return m_criticalSection; }

        // IDeviceNotify
        virtual void OnDeviceLost();
        virtual void OnDeviceRestored();

        void SetUseCustomRenderer(bool use);

    private:
        void ProcessInput();
        void Update();
        bool Render();

        // Cached pointer to device resources.
        std::shared_ptr<DX::DeviceResources> m_deviceResources;

        // Content renderers.
        std::unique_ptr<CharacterFormattingDemoRenderer> m_characterFormattingDemoRenderer;

        Windows::Foundation::IAsyncAction^ m_renderLoopWorker;
        Concurrency::critical_section m_criticalSection;

        // Rendering loop timer.
        DX::StepTimer m_timer;

        // Track current input pointer position.
        float m_pointerLocationX;
    };
}